from bzrlib import branch, tests
from bzrlib.tests import scenarios, TestCaseWithTransport
from bzrlib.plugins.colo.colocated import (NaiveColocatedWorkspace,
                                           CoreColocatedWorkspace,
                                          )

load_tests = scenarios.load_tests_apply_scenarios

class TestWorkspaceInterface(TestCaseWithTransport):

    scenarios = [('workspace_classes', {'workspace_class': ws}) for ws in
                 [NaiveColocatedWorkspace,
                  CoreColocatedWorkspace,]
                ]

    # set from the scenario
    workspace_class = None

    def setUp(self):
        super(TestWorkspaceInterface, self).setUp()
        if self.workspace_class == NaiveColocatedWorkspace:
            from bzrlib.plugins.colo.commands import init_workspace
            init_workspace(u'.', outf=tests.StringIOWrapper())
        elif self.workspace_class == CoreColocatedWorkspace:
            repo = self.make_repository('.', format='development-colo')
            target_branch = repo.bzrdir.create_branch(name='trunk')
            branch.BranchReferenceFormat().initialize(
                repo.bzrdir, target_branch=target_branch)
            repo.bzrdir.create_workingtree()
        self.workspace = self.workspace_class.open()
    
    def test_open(self):
        self.workspace.open()

    def test_url_for_name(self):
        self.assertEndsWith(self.workspace.url_for_name('trunk'), 'trunk')

    def test_branches(self):
        self.assertLength(1, [b for b in self.workspace.branches()])

    def test_repo_transport(self):
        self.assertNotContainsRe(self.workspace.repo_transport().base, 'trunk')

    def test_repository(self):
        self.assertNotContainsRe(self.workspace.repository().base, 'trunk')

    def test_branch_with_name(self):
        self.assertContainsRe(self.workspace.branch_with_name('trunk').base,
                              'trunk')

    def test_current_branch(self):
        self.assertContainsRe(self.workspace.current_branch().base,
                              'trunk')

    def test_current_branch_name(self):
        self.assertEqual(self.workspace.current_branch_name(), 'trunk')

    def test_is_pipe(self):
        self.assertFalse(self.workspace.is_pipe('trunk'))
    
