# Copyright 2011 Neil Martinsen-Burrell

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from bzrlib.plugins.colo.colocated import (
    ColocatedWorkspace,
    CoreColocatedWorkspace,
    )
from bzrlib.plugins.colo.commands import (init_workspace,
                                          COLOCATED_STAGING,
                                         )
from bzrlib.plugins.colo.tests.script import ScriptTestCase

from bzrlib.lazy_import import lazy_import
lazy_import(globals(), '''
from bzrlib import (branch,
                    bzrdir,
                    directory_service,
                    errors,
                    tests,
                    osutils,
                    urlutils,
                   )
''')

class TestCoreColocatedWorkspace(ScriptTestCase):

    def setUp(self):
        super(TestCoreColocatedWorkspace, self).setUp()
        tree = self.make_branch_and_tree('.', format='development-colo')
        self.bzrdir = tree.bzrdir
        self.workspace = ColocatedWorkspace.open()

    def test_detection(self):
        self.assertTrue(isinstance(self.workspace, CoreColocatedWorkspace))

    def test_url_for_name(self):
        self.assertStartsWith(self.workspace.url_for_name('test'), 'file:///')
        self.assertEndsWith(self.workspace.url_for_name('test'), ',branch=test')
        self.assertNotContainsRe(self.workspace.url_for_name('test'), r'\.bzr')
        self.assertEndsWith(self.workspace.url_for_name(''), '/work')

    def test_branches(self):
        branches = [b for b in self.workspace.branches()]
        self.assertEqual(len(branches), 1)

    def test_branch_names(self):
        branch_names = [b for b in self.workspace.branch_names()]
        self.assertEqual(len(branch_names), 1)

    def test_current_branch_name_none(self):
        self.assertEqual(self.workspace.current_branch_name(), None)

    def test_current_branch_name(self):
        repo = self.make_repository('branch-1', format='development-colo')
        target_branch = repo.bzrdir.create_branch(name='foo')
        branch.BranchReferenceFormat().initialize(
            repo.bzrdir, target_branch=target_branch)
        repo.bzrdir.create_workingtree()
        branch_workspace = ColocatedWorkspace.open('branch-1')
        self.assertEqual(branch_workspace.current_branch_name(), 'foo')

    def test_current_branch(self):
        repo = self.make_repository('branch-1', format='development-colo')
        target_branch = repo.bzrdir.create_branch(name='foo')
        branch.BranchReferenceFormat().initialize(
            repo.bzrdir, target_branch=target_branch)
        repo.bzrdir.create_workingtree()
        branch_workspace = ColocatedWorkspace.open('branch-1')
        self.assertEqual(branch_workspace.current_branch().base,
                         target_branch.base)
        

class TestColoInitNative(ScriptTestCase):

    def test_colo_init_native(self):
        self.run_script("""
$ bzr colo-init --native
$ bzr colo-branches
* trunk
""")

    def test_colo_init_native_no_trees(self):
        self.run_script("""
$ bzr colo-init --native --no-tree
2>bzr: ERROR: Native workspaces must have trees
""")
